@@ -1,17 +1,23 @@
 import dataclasses
 from dataclasses import InitVar
 from typing import List, Final, Optional
+
+from dumbo_utils.primitives import PrivateKey
 from lxml import html
 
 import typeguard
 from selenium import webdriver
-from selenium.common import WebDriverException, NoSuchElementException
+from selenium.common import WebDriverException, NoSuchElementException, ElementNotSelectableException, \
+    ElementNotVisibleException
 from selenium.webdriver import Keys
 from selenium.webdriver.common.by import By
+from selenium.webdriver.support import expected_conditions
+from selenium.webdriver.support.wait import WebDriverWait
 
 from dumbo_esse3.primitives import Course, Username, Password, Exam, Student, StudentThesisState, CdL, \
     ExamDescription, ExamNotes, ExamType, DateTime, Register, NumberOfHours, Semester, RegisterActivity, \
-    ActivityTitle, ActivityType, GraduationDay, StudentGraduation
+    ActivityTitle, ActivityType, GraduationDay, StudentGraduation, Committee, CommitteeValuation, CommitteeName, \
+    CommitteePart, FiscalCode
 from dumbo_utils.validation import validate
 
 ESSE3_SERVER = "https://unical.esse3.cineca.it"
@@ -22,6 +28,7 @@ URLs: Final = {
     "thesis_list": f'{ESSE3_SERVER}/auth/docente/Graduation/LaureandiAssegnati.do?menu_opened_cod=menu_link-navbox_docenti_Conseguimento_Titolo',
     "register_list": f'{ESSE3_SERVER}/auth/docente/RegistroDocente/Home.do?menu_opened_cod=menu_link-navbox_docenti_Registro',
     "graduation_day_list": f'{ESSE3_SERVER}/auth/docente/Graduation/ElencoSeduteLaurea.do',
+    "committee_list": f'{ESSE3_SERVER}/auth/Admission/ElencoCommissioni.do?menu_opened_cod=menu_link-navbox_docenti_Commissioni',
 }
 
 
@@ -37,15 +44,15 @@ def change_esse3_server(url):
 @typeguard.typechecked
 @dataclasses.dataclass(frozen=True)
 class Esse3Wrapper:
-    key: InitVar[object]
+    key: InitVar[PrivateKey]
     username: InitVar[Username]
     password: InitVar[Password]
     debug: bool = dataclasses.field(default=False)
     driver: webdriver.Chrome = dataclasses.field(default_factory=webdriver.Chrome)
-    __key = object()
+    __key = PrivateKey()
 
-    def __post_init__(self, key: object, username: Username, password: Password):
-        validate('key', key, equals=self.__key, help_msg="Can only be instantiated using a factory method")
+    def __post_init__(self, key: PrivateKey, username: Username, password: Password):
+        self.__key.validate(key)
         self.maximize()
         self.__login(username, password)
 
@@ -95,8 +102,13 @@ class Esse3Wrapper:
     def maximize(self) -> None:
         self.driver.maximize_window()
 
+    def __check_expiring_token(self):
+        if self.driver.find_elements(By.XPATH, "//strong[text() = 'Certificati in scadenza']"):
+            self.driver.find_element(By.XPATH, "//a[text() = 'Prosegui']").send_keys(Keys.RETURN)
+
     def fetch_courses(self) -> List[Course]:
         self.driver.get(URLs["course_list"])
+        self.__check_expiring_token()
         rows = self.driver.find_elements(By.XPATH, "//tr[@class='detail_table'][td//input[@src = 'images/sostenuta.gif']]")
         res = []
         for idx, row in enumerate(rows):
@@ -106,6 +118,7 @@ class Esse3Wrapper:
 
     def fetch_exams(self, course: Course) -> List[Exam]:
         self.driver.get(URLs["course_list"])
+        self.__check_expiring_token()
         self.driver.find_element(By.XPATH, f"//tr[td = '{course}']/td//input[@src = 'images/sostenuta.gif']").send_keys(Keys.RETURN)
         exams = self.driver.find_elements(By.XPATH, '//tr[@class="detail_table"]')
         return list(sorted([Exam.of(
@@ -115,6 +128,7 @@ class Esse3Wrapper:
 
     def fetch_students(self, course: Course, exam: DateTime) -> List[Student]:
         self.driver.get(URLs["course_list"])
+        self.__check_expiring_token()
         self.driver.find_element(By.XPATH, f"//tr[td = '{course}']/td//input[@src = 'images/sostenuta.gif']").send_keys(Keys.RETURN)
 
         self.driver.find_element(By.XPATH, f"//tr[normalize-space(td/text()) = '{exam}']//input[@src='images/defAppStudent.gif']").send_keys(Keys.RETURN)
@@ -134,6 +148,7 @@ class Esse3Wrapper:
     def add_exam(self, course: Course, exam: DateTime, exam_type: ExamType, description: ExamDescription,
                  notes: ExamNotes) -> None:
         self.driver.get(URLs["course_list"])
+        self.__check_expiring_token()
         self.driver.find_element(By.XPATH, f"//tr[td = '{course}']/td//input[@src = 'images/sostenuta.gif']").send_keys(Keys.RETURN)
 
         self.driver.find_element(By.XPATH, '//input[@type = "submit"][@name = "new_pf"]').send_keys(Keys.RETURN)
@@ -311,6 +326,8 @@ class Esse3Wrapper:
             f"//table[@id = 'seduteAperte']/tbody/tr/td[text() = '{graduation_day}']/../td/a"
         ).send_keys(Keys.RETURN)
 
+        # PAGINATION MUST BE HANDLED AS FOR COMMITTEE EVALUATIONS
+
         html_document = html.fromstring(self.driver.page_source)
         rows = html_document.xpath('//table[@id="elencoLaureandi"]/tbody/tr')
         student_to_url = {
@@ -380,6 +397,89 @@ class Esse3Wrapper:
             if not dry_run:
                 self.driver.find_element(By.ID, 'grad-dettLau-btnSubmit').send_keys(Keys.RETURN)
 
+    def fetch_committees(self) -> List[Committee]:
+        self.driver.get(URLs["committee_list"])
+        rows = self.driver.find_elements(By.XPATH, "//table[@id = 'tableElencoCommissioni']/tbody/tr")
+        committee = [(
+            element.find_element(By.XPATH, "td[1]").text,
+            element.find_element(By.XPATH, "td[3]/a").get_attribute("href"),
+        ) for element in rows]
+        res: Final = []
+        for name, link in committee:
+            committee_name = CommitteeName(name)
+            self.driver.get(link)
+            rows = self.driver.find_elements(By.XPATH, "//table[@id = 'tableElencoConcTurni']/tbody/tr/td[2]")
+            res.extend(Committee(committee_name, CommitteePart(row.text)) for row in rows)
+        return res
+
+    def upload_committee_valuations(
+            self,
+            committee: Committee,
+            valuations: List[CommitteeValuation],
+            dry_run: bool = False,
+    ) -> None:
+        validate("at least one valuation", valuations, min_len=1, help_msg="No valuation was provided")
+
+        self.driver.get(URLs["committee_list"])
+        self.driver.find_element(
+            By.XPATH,
+            f"//table[@id = 'tableElencoCommissioni']/tbody/tr[td[1] = '{committee.name}']/td[3]/a"
+        ).send_keys(Keys.RETURN)
+        self.driver.find_element(
+            By.XPATH,
+            f"//table[@id = 'tableElencoConcTurni']/tbody/tr[td[2] = '{committee.part}']/td[4]/a"
+        ).send_keys(Keys.RETURN)
+
+        ignore_list = [ElementNotVisibleException, ElementNotSelectableException]
+        wait = WebDriverWait(self.driver, timeout=10, poll_frequency=1, ignored_exceptions=ignore_list)
+        wait.until(expected_conditions.presence_of_element_located(
+            (By.XPATH, "//table[@id='tableElencoIscritti']/tfoot/tr"))
+        )
+
+        self.driver.execute_script("""
+            Object.keys(localStorage).forEach(key => {
+                if (key.endsWith(":paging")) {
+                    localStorage.setItem(key, '{"current": 1,"size": 999999}');
+                }
+            });
+            console.log(Object.keys(localStorage));
+            console.log(Object.values(localStorage));
+        """)
+        self.driver.refresh()
+
+        fiscal_code_to_valuation = {valuation.fiscal_code: valuation for valuation in valuations}
+
+        html_document = html.fromstring(self.driver.page_source)
+        fiscal_code_to_url = {
+            FiscalCode(row.xpath("td[3]")[0].text): ESSE3_SERVER + '/' + row.xpath("td[5]/a/@href")[0]
+            for row in html_document.xpath('//table[@id="tableElencoIscritti"]/tbody/tr')
+        }
+
+        validate("valuations", fiscal_code_to_valuation.keys(), equals=fiscal_code_to_url.keys(),
+                 help_msg="Candidates don't match")
+
+        self.driver.get(fiscal_code_to_url[valuations[0].fiscal_code])
+        for valuation in valuations:
+            url = fiscal_code_to_url[valuation.fiscal_code]
+            self.driver.get(url)
+
+            self.__replace_content(
+                self.driver.find_element(By.ID, 'esito'),
+                str(valuation.score)
+            )
+            self.driver.find_element(By.ID, 'valutazioneCompletata1').send_keys(Keys.SPACE)
+            self.__replace_content(
+                self.driver.find_element(By.XPATH, '//div[@id = "nota-div"]//textarea'),
+                str(valuation.notes)
+            )
+            self.__replace_content(
+                self.driver.find_element(By.ID, 'numBusta'),
+                str(valuation.envelope_number) if valuation.envelope_number is not None else ""
+            )
+
+            if not dry_run:
+                self.driver.find_element(By.ID, 'btnSubmit').send_keys(Keys.RETURN)
+
     @staticmethod
     def __replace_content(element, content):
         element.clear()
